pub struct RecordingMap<K, V> { /* private fields */ }Expand description
A RecordingMap that records read requests to the underlying key-value map.
The data recorder is used to generate a proof for read requests.
The RecordingMap is composed of three parts:
data: which contains the current set of key-value pairs in the map.updates: which tracks keys for which values have been changed since the map was instantiated. updates include both insertions, removals and updates of values under existing keys.trace: which contains the key-value pairs from the original data which have been accesses since the map was instantiated.
Implementations
Sourceimpl<K, V> RecordingMap<K, V>
impl<K, V> RecordingMap<K, V>
Sourcepub fn new(init: impl IntoIterator<Item = (K, V)>) -> RecordingMap<K, V>
pub fn new(init: impl IntoIterator<Item = (K, V)>) -> RecordingMap<K, V>
Returns a new RecordingMap instance initialized with the provided key-value pairs. (BTreeMap).
pub fn inner(&self) -> &BTreeMap<K, V>
Sourcepub fn finalize(self) -> (BTreeMap<K, V>, BTreeMap<K, V>)
pub fn finalize(self) -> (BTreeMap<K, V>, BTreeMap<K, V>)
Consumes the RecordingMap and returns a (BTreeMap, BTreeMap) tuple. The first
element of the tuple is a map that represents the state of the map at the time .finalize()
is called. The second element contains the key-value pairs from the initial data set that
were read during recording.