SecretKey

Struct SecretKey 

pub struct SecretKey { /* private fields */ }
Represents the secret key for Falcon DSA.

The secret key is a quadruple [[g, -f], [G, -F]] of polynomials with integer coefficients. Each polynomial is of degree at most N = 512 and computations with these polynomials is done modulo the monic irreducible polynomial ϕ = x^N + 1. The secret key is a basis for a lattice and has the property of being short with respect to a certain norm and an upper bound appropriate for a given security parameter. The public key on the other hand is another basis for the same lattice and can be described by a single polynomial h with integer coefficients modulo ϕ. The two keys are related by the following relation:

  1. h = g /f [mod ϕ][mod p]
  2. f.G - g.F = p [mod ϕ]

where p = 12289 is the Falcon prime. Equation 2 is called the NTRU equation. The secret key is generated by first sampling a random pair (f, g) of polynomials using an appropriate distribution that yields short but not too short polynomials with integer coefficients modulo ϕ. The NTRU equation is then used to find a matching pair (F, G). The public key is then derived from the secret key using equation 1.

To allow for fast signature generation, the secret key is pre-processed into a more suitable form, called the LDL tree, and this allows for fast sampling of short vectors in the lattice using Fast Fourier sampling during signature generation (ffSampling algorithm 11 in 1).

Implementations



impl SecretKey

pub fn new() -> SecretKey

Generates a secret key from OS-provided randomness.

pub fn with_rng<R>(rng: &mut R) -> SecretKey
where R: Rng,

Generates a secret_key using the provided random number generator Rng.

pub fn short_lattice_basis(&self) -> &[Polynomial<i16>; 4]

Returns the polynomials of the short lattice basis of this secret key.

pub fn public_key(&self) -> PublicKey

Returns the public key corresponding to this secret key.

pub fn tree(&self) -> &LdlTree

Returns the LDL tree associated to this secret key.

pub fn sign(&self, message: Word) -> Signature

Signs a message with this secret key.

pub fn sign_with_rng<R>(&self, message: Word, rng: &mut R) -> Signature
where R: Rng,

Signs a message with the secret key relying on the provided randomness generator.