pub struct BatchMerkleProof<H>where
H: Hasher,{
pub nodes: Vec<Vec<<H as Hasher>::Digest>>,
pub depth: u8,
}Expand description
Multiple Merkle proofs aggregated into a single proof.
The aggregation is done in a way which removes all duplicate internal nodes, and thus, it is possible to achieve non-negligible compression as compared to naively concatenating individual Merkle proofs. The algorithm is for aggregation is a variation of Octopus.
Fields
nodes: Vec<Vec<<H as Hasher>::Digest>>Hashes of Merkle Tree proof values above the leaf layer
depth: u8Depth of the leaves
Implementations
Sourceimpl<H> BatchMerkleProof<H>where
H: Hasher,
impl<H> BatchMerkleProof<H>where
H: Hasher,
Sourcepub fn from_single_proofs(
proofs: &[(<H as Hasher>::Digest, Vec<<H as Hasher>::Digest>)],
indexes: &[usize],
) -> BatchMerkleProof<H>
pub fn from_single_proofs( proofs: &[(<H as Hasher>::Digest, Vec<<H as Hasher>::Digest>)], indexes: &[usize], ) -> BatchMerkleProof<H>
Constructs a batch Merkle proof from collection of single Merkle proofs.
Panics
Panics if:
- No proofs have been provided (i.e.,
proofsis an empty slice). - Number of proofs is not equal to the number of indexes.
- Not all proofs have the same length.
Sourcepub fn get_root(
&self,
indexes: &[usize],
leaves: &[<H as Hasher>::Digest],
) -> Result<<H as Hasher>::Digest, MerkleTreeError>
pub fn get_root( &self, indexes: &[usize], leaves: &[<H as Hasher>::Digest], ) -> Result<<H as Hasher>::Digest, MerkleTreeError>
Computes a node to which all Merkle proofs aggregated in this proof resolve.
Errors
Returns an error if:
- No indexes were provided (i.e.,
indexesis an empty slice). - Any of the specified
indexesis greater than or equal to the number of leaves in the tree for which this batch proof was generated. - List of indexes contains duplicates.
- The proof does not resolve to a single root.
Sourcepub fn into_openings(
self,
leaves: &[<H as Hasher>::Digest],
indexes: &[usize],
) -> Result<Vec<(<H as Hasher>::Digest, Vec<<H as Hasher>::Digest>)>, MerkleTreeError>
pub fn into_openings( self, leaves: &[<H as Hasher>::Digest], indexes: &[usize], ) -> Result<Vec<(<H as Hasher>::Digest, Vec<<H as Hasher>::Digest>)>, MerkleTreeError>
Computes the uncompressed individual Merkle proofs which aggregate to this batch proof.
Errors
Returns an error if:
- No indexes were provided (i.e.,
indexesis an empty slice). - Number of provided indexes does not match the number of leaf nodes in the proof.