pub enum ExecutionError {
Show 41 variants
AdviceMapKeyNotFound {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
key: [BaseElement; 4],
},
AdviceMapKeyAlreadyPresent {
key: [BaseElement; 4],
prev_values: Vec<BaseElement>,
new_values: Vec<BaseElement>,
},
AdviceStackReadFailed {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
row: RowIndex,
},
CallInSyscall(&'static str),
CallerNotInSyscall,
CircularExternalNode(RpoDigest),
CycleLimitExceeded(u32),
DecoratorNotFoundInForest {
decorator_id: DecoratorId,
},
DivideByZero {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
clk: RowIndex,
},
DynamicNodeNotFound {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
digest: RpoDigest,
},
EventError {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
error: Box<dyn Error + Send + Sync>,
},
Ext2InttError(Ext2InttError),
FailedAssertion {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
clk: RowIndex,
err_code: BaseElement,
err_msg: Option<Arc<str>>,
},
FailedToExecuteProgram(&'static str),
InvalidFmpValue(BaseElement, BaseElement),
InvalidFriDomainSegment(u64),
InvalidFriLayerFolding(QuadExtension<BaseElement>, QuadExtension<BaseElement>),
InvalidStackDepthOnReturn {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
depth: usize,
},
InvalidMerkleTreeDepth {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
depth: BaseElement,
},
InvalidMerkleTreeNodeIndex {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
depth: BaseElement,
index: BaseElement,
},
LogArgumentZero {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
clk: RowIndex,
},
MalformedSignatureKey {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
key_type: &'static str,
},
MalformedMastForestInHost {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
root_digest: RpoDigest,
},
MastNodeNotFoundInForest {
node_id: MastNodeId,
},
MemoryError(MemoryError),
NoMastForestWithProcedure {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
root_digest: RpoDigest,
},
MerklePathVerificationFailed {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
value: [BaseElement; 4],
index: BaseElement,
root: RpoDigest,
err_code: BaseElement,
err_msg: Option<Arc<str>>,
},
MerkleStoreLookupFailed {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
err: MerkleError,
},
MerkleStoreMergeFailed {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
err: MerkleError,
},
MerkleStoreUpdateFailed {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
err: MerkleError,
},
NotBinaryValueIf {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
value: BaseElement,
},
NotBinaryValueOp {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
value: BaseElement,
},
NotBinaryValueLoop {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
value: BaseElement,
},
NotU32Value {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
value: BaseElement,
err_code: BaseElement,
},
OutputStackOverflow(usize),
ProgramAlreadyExecuted,
ProverError(ProverError),
SmtNodeNotFound {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
node: [BaseElement; 4],
},
SmtNodePreImageNotValid {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
node: [BaseElement; 4],
preimage_len: usize,
},
SyscallTargetNotInKernel {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
proc_root: RpoDigest,
},
AceChipError {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
error: AceError,
},
}Variants
AdviceMapKeyNotFound
AdviceMapKeyAlreadyPresent
AdviceStackReadFailed
CallInSyscall(&'static str)
This error is caught by the assembler, so we don’t need diagnostics here.
CallerNotInSyscall
This error is caught by the assembler, so we don’t need diagnostics here.
CircularExternalNode(RpoDigest)
CycleLimitExceeded(u32)
DecoratorNotFoundInForest
Fields
decorator_id: DecoratorIdDivideByZero
DynamicNodeNotFound
EventError
Ext2InttError(Ext2InttError)
FailedAssertion
FailedToExecuteProgram(&'static str)
InvalidFmpValue(BaseElement, BaseElement)
InvalidFriDomainSegment(u64)
InvalidFriLayerFolding(QuadExtension<BaseElement>, QuadExtension<BaseElement>)
InvalidStackDepthOnReturn
InvalidMerkleTreeDepth
InvalidMerkleTreeNodeIndex
LogArgumentZero
MalformedSignatureKey
MalformedMastForestInHost
MastNodeNotFoundInForest
Fields
node_id: MastNodeIdMemoryError(MemoryError)
NoMastForestWithProcedure
MerklePathVerificationFailed
Fields
label: SourceSpansource_file: Option<Arc<SourceFile>>value: [BaseElement; 4]index: BaseElementerr_code: BaseElementMerkleStoreLookupFailed
MerkleStoreMergeFailed
MerkleStoreUpdateFailed
NotBinaryValueIf
NotBinaryValueOp
NotBinaryValueLoop
NotU32Value
Fields
label: SourceSpansource_file: Option<Arc<SourceFile>>value: BaseElementerr_code: BaseElementOutputStackOverflow(usize)
ProgramAlreadyExecuted
ProverError(ProverError)
SmtNodeNotFound
SmtNodePreImageNotValid
SyscallTargetNotInKernel
AceChipError
Implementations
Sourceimpl ExecutionError
impl ExecutionError
pub fn advice_map_key_not_found( key: [BaseElement; 4], err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn advice_stack_read_failed( row: RowIndex, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn divide_by_zero( clk: RowIndex, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn dynamic_node_not_found( digest: RpoDigest, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn event_error( error: Box<dyn Error + Send + Sync>, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn failed_assertion( clk: RowIndex, err_code: BaseElement, err_msg: Option<Arc<str>>, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn invalid_merkle_tree_depth( depth: BaseElement, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn invalid_merkle_tree_node_index( depth: BaseElement, index: BaseElement, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn invalid_stack_depth_on_return( depth: usize, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn log_argument_zero( clk: RowIndex, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn malfored_mast_forest_in_host( root_digest: RpoDigest, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn malformed_signature_key( key_type: &'static str, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn merkle_path_verification_failed( value: [BaseElement; 4], index: BaseElement, root: RpoDigest, err_code: BaseElement, err_msg: Option<Arc<str>>, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn merkle_store_lookup_failed( err: MerkleError, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
Sourcepub fn merkle_store_merge_failed(
err: MerkleError,
err_ctx: &ErrorContext<'_, impl MastNodeExt>,
) -> ExecutionError
pub fn merkle_store_merge_failed( err: MerkleError, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
Note: This error currently never occurs, since MerkleStore::merge_roots() never fails.
pub fn merkle_store_update_failed( err: MerkleError, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn no_mast_forest_with_procedure( root_digest: RpoDigest, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn not_binary_value_if( value: BaseElement, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn not_binary_value_op( value: BaseElement, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn not_binary_value_loop( value: BaseElement, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn not_u32_value( value: BaseElement, err_code: BaseElement, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn smt_node_not_found( node: [BaseElement; 4], err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn smt_node_preimage_not_valid( node: [BaseElement; 4], preimage_len: usize, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn syscall_target_not_in_kernel( proc_root: RpoDigest, err_ctx: &ErrorContext<'_, impl MastNodeExt>, ) -> ExecutionError
pub fn failed_arithmetic_evaluation( err_ctx: &ErrorContext<'_, impl MastNodeExt>, error: AceError, ) -> ExecutionError