pub enum VerifierError {
InconsistentBaseField,
UnsupportedFieldExtension(usize),
ProofDeserializationError(String),
RandomCoinError,
InconsistentOodConstraintEvaluations,
TraceQueryDoesNotMatchCommitment,
ConstraintQueryDoesNotMatchCommitment,
QuerySeedProofOfWorkVerificationFailed,
FriVerificationFailed(VerifierError),
InsufficientConjecturedSecurity(u32, u32),
InsufficientProvenSecurity(u32, u32),
UnacceptableProofOptions,
}Expand description
Represents an error returned by the verifier during an execution of the protocol.
Variants
InconsistentBaseField
This error occurs when base field read by a verifier from a proof does not match the base field of AIR with which the verifier was instantiated.
UnsupportedFieldExtension(usize)
This error occurs when the base field in which the proof was generated does not support field extension of degree specified by the proof.
ProofDeserializationError(String)
This error occurs when a verifier cannot deserialize the specified proof.
RandomCoinError
This error occurs when a verifier fails to draw a random value from a random coin within a specified number of tries.
InconsistentOodConstraintEvaluations
This error occurs when constraints evaluated over out-of-domain trace rows do not match evaluations of the constraint composition polynomial at the out-of-domain point.
TraceQueryDoesNotMatchCommitment
This error occurs when the batch opening proof fails to verify for trace queries.
ConstraintQueryDoesNotMatchCommitment
This error occurs when the batch opening proof fails to verify for constraint evaluation queries.
QuerySeedProofOfWorkVerificationFailed
This error occurs when the proof-of-work nonce hashed with the current state of the public coin resolves to a value which does not meet the proof-of-work threshold specified by the
FriVerificationFailed(VerifierError)
This error occurs when the DEEP composition polynomial evaluations derived from trace and constraint evaluation queries do not represent a polynomial of the degree expected by the verifier.
InsufficientConjecturedSecurity(u32, u32)
This error occurs when the parameters, that were used to generate the proof, do not provide a conjectured security level greater than or equal to the conjectured security level expected by the verifier.
InsufficientProvenSecurity(u32, u32)
This error occurs when the parameters, that were used to generate the proof, do not provide a proven security level greater than or equal to the proven security level expected by the verifier.
UnacceptableProofOptions
This error occurs when the parameters, that were used to generate the proof, do not match any of the set of parameters expected by the verifier.