macro_rules! build_debug_test {
($($params:tt)+) => { ... };
}Expand description
Returns a Test struct in debug mode from the provided source string and any specified stack and advice inputs.
Parameters are expected in the following order:
source, stack_inputs (optional), advice_stack (optional), merkle_store (optional)
source: a well-formed source string.stack_inputs(optional): the initial inputs which must be at the top of the stack before executing thesource. Stack inputs can be provided independently without any advice inputs.advice_stack(optional): the initial advice stack values. When provided,stack_inputsandmerkle_storeare also expected.merkle_store(optional): the initial merkle set values. When provided,stack_inputsandadvice_stackare also expected.
NOTE: use miden_core_lib::tests::build_debug_test to include the core library in the test.