build_test_by_mode

Macro build_test_by_mode 

macro_rules! build_test_by_mode {
    ($in_debug_mode:expr, $source:expr) => { ... };
    ($in_debug_mode:expr, $source:expr, $stack_inputs:expr) => { ... };
    ($in_debug_mode:expr, $source:expr, $stack_inputs:expr, $advice_stack:expr) => { ... };
    (
        $in_debug_mode:expr,
        $source:expr,
        $stack_inputs:expr,
        $advice_stack:expr,
        $advice_merkle_store:expr
    ) => { ... };
    (
        $in_debug_mode:expr,
        $source:expr,
        $stack_inputs:expr,
        $advice_stack:expr,
        $advice_merkle_store:expr,
        $advice_map:expr
    ) => { ... };
}
Returns a Test struct in the specified debug or non-debug mode using the provided source string and any specified stack and advice inputs.

Parameters start with a boolean flag, in_debug_mode, specifying whether the test is built in debug or non-debug mode. After that, they match the parameters of build_test and build_debug_test macros.

This macro is an internal test builder, and is not intended to be called directly from tests. Instead, the build_test and build_debug_test wrappers should be used.