macro_rules! expect_assembly_error {
($test:expr, $(|)? $( $pattern:pat_param )|+ $( if $guard: expr )? $(,)?) => { ... };
}Expand description
Asserts that running the given assembler test will result in the expected error.
macro_rules! expect_assembly_error {
($test:expr, $(|)? $( $pattern:pat_param )|+ $( if $guard: expr )? $(,)?) => { ... };
}Asserts that running the given assembler test will result in the expected error.