macro_rules! expect_exec_error_matches {
($test:expr, $(|)? $( $pattern:pat_param )|+ $( if $guard: expr )? $(,)?) => { ... };
}Expand description
Asserts that running the given execution test will result in the expected error.
macro_rules! expect_exec_error_matches {
($test:expr, $(|)? $( $pattern:pat_param )|+ $( if $guard: expr )? $(,)?) => { ... };
}Asserts that running the given execution test will result in the expected error.