prng_array

Function prng_array 

pub fn prng_array<R, const N: usize>(seed: [u8; 32]) -> [R; N]
where R: Randomizable + Debug,
Returns an array of value of the specified type and the specified length generated pseudo-randomly from the specified seed.

Panics

Panics if:

  • A valid value requires at over 32 bytes.
  • A valid value could not be generated after 1000 tries.