Function random_felt
pub fn random_felt() -> FeltExpand description
Generates a random field element for testing purposes.
This function is only available with the std feature.
pub fn random_felt() -> FeltGenerates a random field element for testing purposes.
This function is only available with the std feature.