Function random_word
pub fn random_word() -> WordExpand description
Generates a random word (4 field elements) for testing purposes.
This function is only available with the std feature.
pub fn random_word() -> WordGenerates a random word (4 field elements) for testing purposes.
This function is only available with the std feature.