Struct SliceReader
pub struct SliceReader<'a> { /* private fields */ }Expand description
Implements ByteReader trait for a slice of bytes.
NOTE: If you are building with the std feature, you should probably prefer std::io::Cursor
instead. However, SliceReader is still useful in no-std environments until stabilization of
the core_io_borrowed_buf feature.
Implementations
impl<'a> SliceReader<'a>
impl<'a> SliceReader<'a>
pub fn new(source: &'a [u8]) -> SliceReader<'a>
pub fn new(source: &'a [u8]) -> SliceReader<'a>
Creates a new slice reader from the specified slice.