Struct Nonce
pub struct Nonce(/* private fields */);Expand description
Nonce of the Falcon signature.
Implementations
impl Nonce
impl Nonce
pub fn deterministic() -> Nonce
pub fn deterministic() -> Nonce
Returns a new deterministic Nonce.
This is used in deterministic signing following 1 and is composed of two parts:
- a byte serving as a version byte,
- a pre-versioned fixed nonce which is the UTF8 encoding of the domain separator “FALCON-POSEIDON2-DET” padded with enough zeros to make it of size 39 bytes.
The usefulness of the notion of versioned fixed nonce is discussed in Section 2.1 in 1.
pub fn from_bytes(nonce_bytes: [u8; 40]) -> Nonce
pub fn from_bytes(nonce_bytes: [u8; 40]) -> Nonce
Returns a Nonce given an array of bytes.
pub fn to_elements(&self) -> [Felt; 8]
pub fn to_elements(&self) -> [Felt; 8]
Converts byte representation of the nonce into field element representation.
Nonce bytes are converted to field elements by taking consecutive 5 byte chunks of the nonce and interpreting them as field elements.