macro_rules! build_op_test {
($op_str:expr) => { ... };
($op_str:expr, $($tail:tt)+) => { ... };
}Expand description
Returns a Test struct in non debug mode from a string of one or more operations and any specified stack and advice inputs.
Parameters are expected in the following order:
source, stack_inputs (optional), advice_stack (optional), merkle_store (optional)
source: a string of one or more operations, e.g. “push.1 push.2”.stack_inputs(optional): the initial inputs which must be at the top of the stack before executing thesource. Stack inputs can be provided independently without any advice inputs.advice_stack(optional): the initial advice stack values. When provided,stack_inputsandmerkle_storeare also expected.merkle_store(optional): the initial merkle set values. When provided,stack_inputsandadvice_stackare also expected.