Felt

Struct Felt 

pub struct Felt(/* private fields */);
A Felt backed by Plonky3’s Goldilocks field element.

Implementations



impl Felt

pub const ORDER: u64 = <Goldilocks as PrimeField64>::ORDER_U64

Order of the field.

pub const ZERO: Felt

pub const ONE: Felt

pub const NUM_BYTES: usize = Goldilocks::NUM_BYTES

The number of bytes which this field element occupies in memory.

pub const fn new(value: u64) -> Felt

Creates a new field element from any u64.

Any u64 value is accepted. No reduction is performed since Goldilocks uses a non-canonical internal representation.

pub fn from_u8(value: u8) -> Felt

pub fn from_u16(value: u16) -> Felt

pub fn from_u32(value: u32) -> Felt

pub fn double(&self) -> Felt

The elementary function double(a) = 2*a.

pub fn square(&self) -> Felt

The elementary function square(a) = a^2.

pub fn exp_u64(&self, power: u64) -> Felt

Exponentiation by a u64 power.

pub fn exp_const_u64<const POWER: u64>(&self) -> Felt

Exponentiation by a small constant power.

pub fn as_canonical_u64(&self) -> u64

Return the representative of element in canonical form which lies in the range 0 <= x < ORDER.

pub fn as_canonical_u64_ct(&self) -> u64

Constant-time equivalent of as_canonical_u64() using the same reduction logic as Plonky3’s Goldilocks implementation.

Methods from Deref<Target = Goldilocks>

pub const TWO_ADIC_GENERATORS: [Goldilocks; 33]