Struct Felt
pub struct Felt(/* private fields */);Expand description
A Felt backed by Plonky3’s Goldilocks field element.
Implementations
impl Felt
impl Felt
pub const ZERO: Felt
pub const ONE: Felt
pub const NUM_BYTES: usize = Goldilocks::NUM_BYTES
pub const NUM_BYTES: usize = Goldilocks::NUM_BYTES
The number of bytes which this field element occupies in memory.
pub const fn new(value: u64) -> Felt
pub const fn new(value: u64) -> Felt
Creates a new field element from any u64.
Any u64 value is accepted. No reduction is performed since Goldilocks uses a
non-canonical internal representation.
pub fn from_u8(value: u8) -> Felt
pub fn from_u16(value: u16) -> Felt
pub fn from_u32(value: u32) -> Felt
pub fn exp_const_u64<const POWER: u64>(&self) -> Felt
pub fn exp_const_u64<const POWER: u64>(&self) -> Felt
Exponentiation by a small constant power.
pub fn as_canonical_u64(&self) -> u64
pub fn as_canonical_u64(&self) -> u64
Return the representative of element in canonical form which lies in the range
0 <= x < ORDER.
pub fn as_canonical_u64_ct(&self) -> u64
pub fn as_canonical_u64_ct(&self) -> u64
Constant-time equivalent of as_canonical_u64() using the same reduction logic as
Plonky3’s Goldilocks implementation.
Methods from Deref<Target = Goldilocks>
pub const TWO_ADIC_GENERATORS: [Goldilocks; 33]