Struct ProcessorState
pub struct ProcessorState<'a> { /* private fields */ }Expand description
A view into the current state of the processor.
This struct provides read access to the processor’s state, including the stack, memory, advice provider, and execution context information.
Implementations
impl<'a> ProcessorState<'a>
impl<'a> ProcessorState<'a>
pub fn advice_provider(&self) -> &AdviceProvider
pub fn advice_provider(&self) -> &AdviceProvider
Returns a reference to the advice provider.
pub fn execution_options(&self) -> &ExecutionOptions
pub fn execution_options(&self) -> &ExecutionOptions
Returns the execution options.
pub fn clock(&self) -> RowIndex
pub fn clock(&self) -> RowIndex
Returns the current clock cycle of a process.
pub fn get_stack_item(&self, pos: usize) -> Felt
pub fn get_stack_item(&self, pos: usize) -> Felt
Returns the value located at the specified position on the stack at the current clock cycle.
This method can access elements beyond the top 16 positions by using the overflow table.
pub fn get_stack_word(&self, start_idx: usize) -> Word
pub fn get_stack_word(&self, start_idx: usize) -> Word
Returns a word starting at the specified element index on the stack.
The word is formed by taking 4 consecutive elements starting from the specified index. For example, start_idx=0 creates a word from stack elements 0-3, start_idx=1 creates a word from elements 1-4, etc.
Stack element N will be at position 0 of the word, N+1 at position 1, N+2 at position 2,
and N+3 at position 3. word[0] corresponds to the top of the stack.
This method can access elements beyond the top 16 positions by using the overflow table. Creating a word does not change the state of the stack.
pub fn get_stack_state(&self) -> Vec<Felt>
pub fn get_stack_state(&self) -> Vec<Felt>
Returns stack state at the current clock cycle. This includes the top 16 items of the stack + overflow entries.
pub fn get_mem_value(&self, ctx: ContextId, addr: u32) -> Option<Felt>
pub fn get_mem_value(&self, ctx: ContextId, addr: u32) -> Option<Felt>
Returns the element located at the specified context/address, or None if the address hasn’t been accessed previously.
pub fn get_mem_word(
&self,
ctx: ContextId,
addr: u32,
) -> Result<Option<Word>, MemoryError>
pub fn get_mem_word( &self, ctx: ContextId, addr: u32, ) -> Result<Option<Word>, MemoryError>
Returns the batch of elements starting at the specified context/address.
Errors
- If the address is not word aligned.
pub fn get_mem_addr_range(
&self,
start_idx: usize,
end_idx: usize,
) -> Result<Range<u32>, MemoryError>
pub fn get_mem_addr_range( &self, start_idx: usize, end_idx: usize, ) -> Result<Range<u32>, MemoryError>
Reads (start_addr, end_addr) tuple from the specified elements of the operand stack ( without modifying the state of the stack), and verifies that memory range is valid.
pub fn get_mem_state(&self, ctx: ContextId) -> Vec<(MemoryAddress, Felt)>
pub fn get_mem_state(&self, ctx: ContextId) -> Vec<(MemoryAddress, Felt)>
Returns the entire memory state for the specified execution context at the current clock cycle.
The state is returned as a vector of (address, value) tuples, and includes addresses which have been accessed at least once.