Struct SourceFile
pub struct SourceFile { /* private fields */ }Expand description
A SourceFile represents a single file stored in a super::SourceManager
Implementations
impl SourceFile
impl SourceFile
pub fn new(
id: SourceId,
lang: SourceLanguage,
uri: Uri,
content: impl Into<Box<str>>,
) -> SourceFile
pub fn new( id: SourceId, lang: SourceLanguage, uri: Uri, content: impl Into<Box<str>>, ) -> SourceFile
Create a new SourceFile from its raw components
pub fn from_raw_parts(id: SourceId, content: SourceContent) -> SourceFile
pub fn from_raw_parts(id: SourceId, content: SourceContent) -> SourceFile
This function is intended for use by super::SourceManager implementations that need to construct a SourceFile from its raw components (i.e. the identifier for the source file and its content).
Since the only entity that should be constructing a [SourceId] is a super::SourceManager, it is only valid to call this function in one of two scenarios:
- You are a super::SourceManager constructing a SourceFile after allocating a [SourceId]
- You pass [
SourceId::default()], i.e. [SourceId::UNKNOWN] for the source identifier. The resulting SourceFile will be valid and safe to use in a context where there isn’t a super::SourceManager present. If there is a source manager in use, then constructing detached SourceFiles is not recommended, because it will make it confusing to determine whether a given SourceFile reference is safe to use.
You should rarely, if ever, fall in camp 2 - but it can be handy in some narrow cases
pub const fn id(&self) -> SourceId
pub const fn id(&self) -> SourceId
Get the [SourceId] associated with this file
pub fn uri(&self) -> &Uri
pub fn uri(&self) -> &Uri
Get the name of this source file
pub fn content(&self) -> &SourceContent
pub fn content(&self) -> &SourceContent
Returns a reference to the underlying [SourceContent]
pub fn content_mut(&mut self) -> &mut SourceContent
pub fn content_mut(&mut self) -> &mut SourceContent
Returns a mutable reference to the underlying [SourceContent]
pub fn line_count(&self) -> usize
pub fn line_count(&self) -> usize
Returns the number of lines in this file
pub fn source_span(&self) -> SourceSpan
pub fn source_span(&self) -> SourceSpan
Returns a [SourceSpan] covering the entirety of this file
pub fn source_slice(&self, span: impl Into<Range<usize>>) -> Option<&str>
pub fn source_slice(&self, span: impl Into<Range<usize>>) -> Option<&str>
Returns a subset of the underlying content as a string slice.
The bounds of the given span are byte indices, not character indices.
Returns None if the given span is out of bounds, or if the bounds do not
fall on valid UTF-8 character boundaries.
pub fn slice(
self: &Arc<SourceFile>,
span: impl Into<Range<u32>>,
) -> SourceFileRef
pub fn slice( self: &Arc<SourceFile>, span: impl Into<Range<u32>>, ) -> SourceFileRef
Returns a [SourceFileRef] corresponding to the bytes contained in the specified span.
pub fn line_column_to_span(
&self,
line: LineNumber,
column: ColumnNumber,
) -> Option<SourceSpan>
pub fn line_column_to_span( &self, line: LineNumber, column: ColumnNumber, ) -> Option<SourceSpan>
Get a [SourceSpan] which points to the first byte of the character at column on line
Returns None if the given line/column is out of bounds for this file.
pub fn location(&self, span: SourceSpan) -> FileLineCol
pub fn location(&self, span: SourceSpan) -> FileLineCol
Get a [FileLineCol] equivalent to the start of the given [SourceSpan]