Struct StackInputs
pub struct StackInputs { /* private fields */ }Expand description
Defines the initial state of the VM’s operand stack.
The first element is at position 0 (top of stack).
Implementations
impl StackInputs
impl StackInputs
pub fn new(values: &[Felt]) -> Result<StackInputs, InputError>
pub fn new(values: &[Felt]) -> Result<StackInputs, InputError>
Returns StackInputs from a list of values.
The first element will be at position 0 (top of stack).
Errors
Returns an error if the number of input values exceeds the allowed maximum.
pub fn try_from_ints<I>(iter: I) -> Result<StackInputs, InputError>where
I: IntoIterator<Item = u64>,
pub fn try_from_ints<I>(iter: I) -> Result<StackInputs, InputError>where
I: IntoIterator<Item = u64>,
Attempts to create stack inputs from an iterator of integers.
Errors
Returns an error if:
- The values do not represent a valid field element.
- Number of values in the iterator exceeds the allowed maximum number of input values.